@@ -1,6 +1,6 @@
 from types import ModuleType
 import importlib, sys
-from ... import context
+from ...context import context
 
 class module(ModuleType):
     def __init__(self, submodules):
