@@ -915,51 +915,51 @@
 #define F_TLOCK 2
 #define F_TEST 3
 #define STAT64_HAS_BROKEN_ST_INO 1
-#define S_IFMT 00170000
-#define S_IFSOCK 0140000
-#define S_IFLNK 0120000
-#define S_IFREG 0100000
-#define S_IFBLK 0060000
-#define S_IFDIR 0040000
-#define S_IFCHR 0020000
-#define S_IFIFO 0010000
-#define S_ISUID 0004000
-#define S_ISGID 0002000
-#define S_ISVTX 0001000
-#define S_IRWXU 00700
-#define S_IRUSR 00400
-#define S_IWUSR 00200
-#define S_IXUSR 00100
-#define S_IRWXG 00070
-#define S_IRGRP 00040
-#define S_IWGRP 00020
-#define S_IXGRP 00010
-#define S_IRWXO 00007
-#define S_IROTH 00004
-#define S_IWOTH 00002
-#define S_IXOTH 00001
-#define S_IREAD 00400
-#define S_IWRITE 00200
-#define S_IEXEC 00100
+#define S_IFMT 0xf000
+#define S_IFSOCK 0xc000
+#define S_IFLNK 0xa000
+#define S_IFREG 0x8000
+#define S_IFBLK 0x6000
+#define S_IFDIR 0x4000
+#define S_IFCHR 0x2000
+#define S_IFIFO 0x1000
+#define S_ISUID 0x800
+#define S_ISGID 0x400
+#define S_ISVTX 0x200
+#define S_IRWXU 0x1c0
+#define S_IRUSR 0x100
+#define S_IWUSR 0x80
+#define S_IXUSR 0x40
+#define S_IRWXG 0x38
+#define S_IRGRP 0x20
+#define S_IWGRP 0x10
+#define S_IXGRP 0x8
+#define S_IRWXO 0x7
+#define S_IROTH 0x4
+#define S_IWOTH 0x2
+#define S_IXOTH 0x1
+#define S_IREAD 0x100
+#define S_IWRITE 0x80
+#define S_IEXEC 0x40
 #define F_LINUX_SPECIFIC_BASE 1024
-#define O_ACCMODE 0003
-#define O_RDONLY 00
-#define O_WRONLY 01
-#define O_RDWR 02
-#define O_CREAT 0100
-#define O_EXCL 0200
-#define O_NOCTTY 0400
-#define O_TRUNC 01000
-#define O_APPEND 02000
-#define O_NONBLOCK 04000
-#define O_NDELAY 04000
-#define O_SYNC 010000
-#define FASYNC 020000
-#define O_DIRECTORY 040000
-#define O_NOFOLLOW 0100000
-#define O_DIRECT 0200000
-#define O_LARGEFILE 0400000
-#define O_NOATIME 01000000
+#define O_ACCMODE 0x3
+#define O_RDONLY 0x0
+#define O_WRONLY 0x1
+#define O_RDWR 0x2
+#define O_CREAT 0x40
+#define O_EXCL 0x80
+#define O_NOCTTY 0x100
+#define O_TRUNC 0x200
+#define O_APPEND 0x400
+#define O_NONBLOCK 0x800
+#define O_NDELAY 0x800
+#define O_SYNC 0x1000
+#define FASYNC 0x2000
+#define O_DIRECTORY 0x4000
+#define O_NOFOLLOW 0x8000
+#define O_DIRECT 0x10000
+#define O_LARGEFILE 0x20000
+#define O_NOATIME 0x40000
 #define F_DUPFD 0
 #define F_GETFD 1
 #define F_SETFD 2
@@ -990,7 +990,7 @@
 #define LOCK_READ 64
 #define LOCK_WRITE 128
 #define LOCK_RW 192
-#define O_ASYNC 020000
+#define O_ASYNC 0x2000
 #define MREMAP_MAYMOVE 1
 #define MREMAP_FIXED 2
 #define PROT_READ 0x1
