@@ -1,7 +1,7 @@
 from types import ModuleType
 import sys, os, re
 from . import internal
-from .. import context
+from ..context import context
 
 class module(ModuleType):
     def __init__(self, name, directory):
@@ -36,7 +36,7 @@ class module(ModuleType):
             path = os.path.join(self._absdir, name)
             if os.path.isdir(path):
                 self._submodules[name] = module(self.__name__ + '.' + name, os.path.join(self._dir, name))
-            elif os.path.isfile(path) and name != '__doc__':
+            elif os.path.isfile(path) and name != '__doc__' and name[0] != '.':
                 funcname, _ext = os.path.splitext(name)
                 if not re.match('^[a-zA-Z][a-zA-Z0-9_]*$', funcname):
                     raise ValueError("found illegal filename, %r" % name)
@@ -102,4 +102,22 @@ class module(ModuleType):
 tether = sys.modules[__name__]
 
 # Create the module structure
-module(__name__, '')
+shellcraft = module(__name__, '')
+
+class LazyImporter:
+    def find_module(self, fullname, path):
+        if not fullname.startswith('pwnlib.shellcraft.'):
+            return None
+
+        parts = fullname.split('.')[2:]
+        cur = shellcraft
+        for part in parts:
+            cur = getattr(cur, part, None)
+            if not isinstance(cur, ModuleType):
+                return None
+
+        return self
+
+    def load_module(self, fullname):
+        return sys.modules[fullname]
+sys.meta_path.append(LazyImporter())
