@@ -1,5 +1,5 @@
 import sys
-from . import term, text, key
+from . import term, text
 from . import keymap as km
 from . import keyconsts as kc
 cursor = text.reverse
@@ -68,7 +68,7 @@ def auto_complete(*_):
         redisplay()
 
 def handle_keypress(trace):
-    global tabs, show_suggestions
+    global tabs
     k = trace[-1]
     if k == '<tab>':
         tabs += 1
@@ -257,6 +257,7 @@ def kill_to_end(*_):
     redisplay()
 
 def delete_char_forward(*_):
+    global buffer_right
     commit_search()
     if buffer_right:
         buffer_right = buffer_right[1:]
@@ -373,7 +374,7 @@ def readline(_size = None, prompt = '', float = False, priority = 10):
                 if eof:
                     return ''
                 else:
-                    buffer = buffer_left + buffer_right
+                    buffer = (buffer_left + buffer_right).encode('utf-8')
                     if buffer:
                         history.insert(0, buffer)
                     return buffer + '\n'
