@@ -10,6 +10,6 @@ just enough for the class to work and possibly code pertaining only to
 that specific kind of tube.
 """
 
-__all__ = ['tube', 'sock', 'remote', 'listen', 'process', 'ssh']
+__all__ = ['tube', 'sock', 'remote', 'listen', 'process', 'serialtube', 'ssh']
 
-from . import tube, sock, remote, listen, process, ssh
+from . import tube, sock, remote, listen, process, serialtube, ssh
