@@ -1,11 +1,44 @@
-from .. import log, log_levels
-from . import tube
-import subprocess, fcntl, os, select
+from .tube import tube
+from ..timeout import Timeout
+from ..util.misc import which
+from ..context import context
+import subprocess, fcntl, os, select, logging
 
-class process(tube.tube):
-    def __init__(self, args, shell = False, executable = None, env = None,
-                 timeout = 'default', log_level = log_levels.INFO):
-        super(process, self).__init__(timeout, log_level)
+log = logging.getLogger(__name__)
+
+class process(tube):
+    r"""
+    Implements a tube which talks to a process on stdin/stdout/stderr.
+
+    Examples:
+
+        >>> context.log_level='error'
+        >>> p = process(which('python2'))
+        >>> p.sendline("print 'Hello world'")
+        >>> p.sendline("print 'Wow, such data'");
+        >>> '' == p.recv(timeout=0.01)
+        True
+        >>> p.shutdown('send')
+        >>> p.proc.stdin.closed
+        True
+        >>> p.connected('send')
+        False
+        >>> p.recvline()
+        'Hello world\n'
+        >>> p.recvuntil(',')
+        'Wow,'
+        >>> p.recvregex('.*data')
+        ' such data'
+        >>> p.recv()
+        '\n'
+        >>> p.recv() # doctest: +ELLIPSIS
+        Traceback (most recent call last):
+        ...
+        EOFError
+    """
+    def __init__(self, args, shell = False, executable = None,
+                 cwd = None, env = None, timeout = Timeout.default):
+        super(process, self).__init__(timeout)
 
         if executable:
             self.program = executable
@@ -16,8 +49,14 @@ class process(tube.tube):
         else:
             log.error("process(): Do not understand the arguments %r" % args)
 
+        # If we specify something not in $PATH, but which exists as a non-executable
+        # file then give an error message.
+        if not which(self.program) and os.path.exists(self.program) and not os.access(self.program, os.X_OK):
+            log.error('%r is not set to executable (chmod +x %s)' % (self.program, self.program))
+
         self.proc = subprocess.Popen(
-            args, shell = shell, executable = executable, env = env,
+            args, shell = shell, executable = executable,
+            cwd = cwd, env = env,
             stdin = subprocess.PIPE, stdout = subprocess.PIPE,
             stderr = subprocess.STDOUT)
         self.stop_noticed = False
@@ -28,7 +67,8 @@ class process(tube.tube):
         fl = fcntl.fcntl(fd, fcntl.F_GETFL)
         fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)
 
-        log.success("Started program %r" % self.program, log_level = self.log_level)
+        log.success("Started program %r" % self.program)
+        log.debug("...with arguments %r" % args)
 
     def kill(self):
         """kill()
@@ -47,7 +87,7 @@ class process(tube.tube):
         self.proc.poll()
         if self.proc.returncode != None and not self.stop_noticed:
             self.stop_noticed = True
-            log.info("Program %r stopped with exit code %d" % (self.program, self.proc.returncode), log_level = self.log_level)
+            log.info("Program %r stopped with exit code %d" % (self.program, self.proc.returncode))
 
         return self.proc.returncode
 
@@ -65,29 +105,29 @@ class process(tube.tube):
         # dead, so we can write a message.
         self.poll()
 
-        if self.proc.stdout.closed:
+        if not self.connected_raw('recv'):
             raise EOFError
 
         if not self.can_recv_raw(self.timeout):
-            return None
+            return ''
 
         # This will only be reached if we either have data,
         # or we have reached an EOF. In either case, it
         # should be safe to read without expecting it to block.
         data = self.proc.stdout.read(numb)
 
-        if data == '':
-            self.proc.stdout.close()
+        if not data:
+            self.shutdown("recv")
             raise EOFError
-        else:
-            return data
+
+        return data
 
     def send_raw(self, data):
         # This is a slight hack. We try to notice if the process is
         # dead, so we can write a message.
         self.poll()
 
-        if self.proc.stdin.closed:
+        if not self.connected_raw('send'):
             raise EOFError
 
         try:
@@ -100,23 +140,40 @@ class process(tube.tube):
         pass
 
     def can_recv_raw(self, timeout):
-        if timeout == None:
-            return select.select([self.proc.stdout], [], []) == ([self.proc.stdout], [], [])
-        else:
-            return select.select([self.proc.stdout], [], [], timeout) == ([self.proc.stdout], [], [])
+        if not self.connected_raw('recv'):
+            return False
 
-    def connected(self):
-        return self.poll() == None
+        try:
+            if timeout == None:
+                return select.select([self.proc.stdout], [], []) == ([self.proc.stdout], [], [])
+
+            return select.select([self.proc.stdout], [], [], timeout) == ([self.proc.stdout], [], [])
+        except ValueError:
+            # Not sure why this isn't caught when testing self.proc.stdout.closed,
+            # but it's not.
+            #
+            #   File "/home/user/pwntools/pwnlib/tubes/process.py", line 112, in can_recv_raw
+            #     return select.select([self.proc.stdout], [], [], timeout) == ([self.proc.stdout], [], [])
+            # ValueError: I/O operation on closed file
+            raise EOFError
+
+    def connected_raw(self, direction):
+        if direction == 'any':
+            return self.poll() == None
+        elif direction == 'send':
+            return not self.proc.stdin.closed
+        elif direction == 'recv':
+            return not self.proc.stdout.closed
 
     def close(self):
         # First check if we are already dead
         self.poll()
 
-        if self.stop_noticed:
+        if not self.stop_noticed:
             try:
                 self.proc.kill()
                 self.stop_noticed = True
-                log.info('Stopped program %r' % self.program, log_level = self.log_level)
+                log.info('Stopped program %r' % self.program)
             except OSError:
                 pass
 
@@ -127,11 +184,11 @@ class process(tube.tube):
 
         return self.proc.stdout.fileno()
 
-    def shutdown(self, direction = "out"):
-        if direction == "out":
+    def shutdown_raw(self, direction):
+        if direction == "send":
             self.proc.stdin.close()
 
-        if direction == "in":
+        if direction == "recv":
             self.proc.stdout.close()
 
         if False not in [self.proc.stdin.closed, self.proc.stdout.closed]:
