@@ -1,13 +1,14 @@
-import socket, errno, select
-from . import tube
-from .. import log
+import socket, errno, select, logging
+from .tube import tube
 
-class sock(tube.tube):
+log = logging.getLogger(__name__)
+
+class sock(tube):
     """Methods available exclusively to sockets."""
 
-    def __init__(self, timeout, log_level):
-        super(sock, self).__init__(timeout, log_level)
-        self.closed = {"in": False, "out": False}
+    def __init__(self, timeout):
+        super(sock, self).__init__(timeout)
+        self.closed = {"recv": False, "send": False}
 
     # Overwritten for better usability
     def recvall(self):
@@ -16,90 +17,89 @@ class sock(tube.tube):
         Receives data until the socket is closed.
         """
 
-        if hasattr(self, 'type') and self.type == socket.SOCK_DGRAM:
+        if getattr(self, 'type', None) == socket.SOCK_DGRAM:
             log.error("UDP sockets does not supports recvall")
         else:
             return super(sock, self).recvall()
 
     def recv_raw(self, numb):
-        if self.closed["in"]:
+        if self.closed["recv"]:
             raise EOFError
 
-        go = True
-        while go:
-            go = False
-
+        while True:
             try:
                 data = self.sock.recv(numb)
+                break
             except socket.timeout:
                 return None
             except IOError as e:
                 if e.errno == errno.EAGAIN:
                     return None
-                elif e.errno == errno.ECONNREFUSED:
-                    self.shutdown("in")
+                elif e.errno in [errno.ECONNREFUSED, errno.ECONNRESET]:
+                    self.shutdown("recv")
                     raise EOFError
                 elif e.errno == errno.EINTR:
-                    go = True
+                    continue
                 else:
                     raise
 
         if data == '':
-            self.shutdown("in")
+            self.shutdown("recv")
             raise EOFError
-        else:
-            return data
+
+        return data
 
     def send_raw(self, data):
-        if self.closed["out"]:
+        if self.closed["send"]:
             raise EOFError
 
         try:
             self.sock.sendall(data)
-        except socket.error as e:
-            if e.message == 'Socket is closed':
-                self.shutdown("out")
-                raise EOFError
-            else:
-                raise
         except IOError as e:
-            if e.errno in [errno.EPIPE, errno.ECONNRESET, errno.ECONNREFUSED]:
-                self.shutdown("out")
+            eof_numbers = [errno.EPIPE, errno.ECONNRESET, errno.ECONNREFUSED]
+            if e.message == 'Socket is closed' or e.errno in eof_numbers:
+                self.shutdown("send")
                 raise EOFError
             else:
                 raise
 
     def settimeout_raw(self, timeout):
-        if not self.sock:
-            return
-
-        if timeout != None and timeout <= 0:
-            self.sock.setblocking(0)
-        else:
-            self.sock.setblocking(1)
+        if getattr(self, 'sock', None):
             self.sock.settimeout(timeout)
 
     def can_recv_raw(self, timeout):
-        if not self.sock or self.closed["in"]:
+        if not self.sock or self.closed["recv"]:
             return False
 
         return select.select([self.sock], [], [], timeout) == ([self.sock], [], [])
 
-    def connected(self):
-        return self.sock != None
+    def connected_raw(self, direction):
+        if not self.sock:
+            return False
+
+        if direction == 'any':
+            return True
+        elif direction == 'recv':
+            return not self.closed['recv']
+        elif direction == 'send':
+            return not self.closed['send']
 
     def close(self):
-        if not self.sock:
+        if not getattr(self, 'sock', None):
             return
 
+        # Call shutdown without triggering another call to close
+        self.closed['hack'] = False
+        self.shutdown('recv')
+        self.shutdown('send')
+        del self.closed['hack']
+
         self.sock.close()
         self.sock = None
-        self.closed["in"]  = True
-        self.closed["out"] = True
         self._close_msg()
 
     def _close_msg(self):
-        log.info('Closed connection to %s port %d' % (self.rhost, self.rport), log_level = self.log_level)
+        log.info('Closed connection to %s port %d' % (self.rhost, self.rport))
 
     def fileno(self):
         if not self.sock:
@@ -107,13 +107,13 @@ class sock(tube.tube):
 
         return self.sock.fileno()
 
-    def shutdown(self, direction = "out"):
+    def shutdown_raw(self, direction):
         if self.closed[direction]:
             return
 
         self.closed[direction] = True
 
-        if direction == "out":
+        if direction == "send":
             try:
                 self.sock.shutdown(socket.SHUT_WR)
             except IOError as e:
@@ -122,7 +122,7 @@ class sock(tube.tube):
                 else:
                     raise
 
-        if direction == "in":
+        if direction == "recv":
             try:
                 self.sock.shutdown(socket.SHUT_RD)
             except IOError as e:
