@@ -1,6 +1,9 @@
+# -*- coding: utf-8 -*-
 import re, base64, random, string
 from . import packing, lists
-from .. import context
+from .cyclic import cyclic_find
+from ..context import context
+from ..term    import text
 
 def unhex(s):
     """unhex(s) -> str
@@ -285,7 +288,7 @@ def xor(*args, **kwargs):
     if len(args) == 0:
         raise ValueError("Must have something to xor")
 
-    strs = [packing.flat(s, word_size = 8, sign = 'unsigned', endianness = 'little') for s in args]
+    strs = [packing.flat(s, word_size = 8, sign = False, endianness = 'little') for s in args]
     strs = [[ord(c) for c in s] for s in strs if s != '']
 
     if strs == []:
@@ -309,11 +312,8 @@ def xor(*args, **kwargs):
 
     return ''.join(get(n) for n in range(cut))
 
-_default_alphabet = ''.join(chr(n) for n in range(256) if n not in [0, 0xa])
-_default_avoid = '\x00\n'
-
-def xor_pair(data, avoid = None):
-    """xor_pair(data, avoid = None) -> None or (str, str)
+def xor_pair(data, avoid = '\x00\n'):
+    """xor_pair(data, avoid = '\\x00\\n') -> None or (str, str)
 
     Finds two strings that will xor into a given string, while only
     using a given alphabet.
@@ -331,7 +331,6 @@ def xor_pair(data, avoid = None):
       ('\\x01\\x01\\x01\\x01', 'udru')
 """
 
-    avoid = avoid or _default_avoid
     alphabet = ''.join(chr(n) for n in range(256) if chr(n) not in avoid)
 
     res1 = ''
@@ -350,19 +349,25 @@ def xor_pair(data, avoid = None):
     return res1, res2
 
 
-def randoms(count, alphabet = None):
-    """randoms(count, alphabet = None) -> str
+def randoms(count, alphabet = string.lowercase):
+    """randoms(count, alphabet = string.lowercase) -> str
 
     Returns a random string of a given length using only the specified alphabet.
 
     Args:
       count (int): The length of the desired string.
-      alphabet: The alphabet of allowed characters. Defaults to all characters except nulls and newlines.
+      alphabet: The alphabet of allowed characters. Defaults to all lowercase characters.
 
     Returns:
-      A random string."""
+      A random string.
 
-    return ''.join(random.sample(alphabet or _default_alphabet, count))
+    Example:
+
+      >>> randoms(10) #doctest: +SKIP
+      'evafjilupm'
+"""
+
+    return ''.join(random.choice(alphabet) for _ in xrange(count))
 
 
 def rol(n, k, word_size = None):
@@ -417,7 +422,7 @@ def isprint(c):
     """isprint(c) -> bool
 
     Return True if a character is printable"""
-    return c in string.ascii_letters + string.digits + string.punctuation
+    return c in string.ascii_letters + string.digits + string.punctuation + ' '
 
 
 def hexii(s, width = 16, skip = True):
@@ -447,56 +452,132 @@ def _hexiichar(c):
     else:
         return "%02x " % ord(c)
 
-def hexdump(s, width = 16, skip = True, hexii = False):
-    """hexdump(s, width = 16, skip = True, hexii = False) -> str
+default_style = {
+    'marker':       text.gray if text.has_gray else text.blue,
+    'nonprintable': text.gray if text.has_gray else text.blue,
+    '00':           text.red,
+    'ff':           text.green,
+}
 
-    Return a hexdump-dump of a string.
+def sequential_lines(a,b):
+    if len(a) != len(b) or len(a) < 4:
+        return False
+
+    all_chars = sorted(set(a+b))
+
+    alphabet = ''
+    if all(a in string.lowercase for a in all_chars):
+        alphabet = string.lowercase
+    if all(a in string.uppercase for a in all_chars):
+        alphabet = string.uppercase
+
+    # Check each set of four
+    for i in range(0, len(a)-3):
+        A = cyclic_find(a[i:i+4], alphabet)
+        B = cyclic_find(b[i:i+4], alphabet)
+        if A+len(a) != B:
+            return False
+    return True
+
+def hexdump_iter(s, width = 16, skip = True, hexii = False, begin = 0,
+                 style = None, highlight = None):
+    """hexdump_iter(s, width = 16, skip = True, hexii = False, begin = 0,
+                    style = {}, highlight = []) -> str generator
+
+    Return a hexdump-dump of a string as a generator of lines.
 
     Args:
       s(str): The string to dump
       width(int): The number of characters per line
       skip(bool): Set to True, if repeated lines should be replaced by a "*"
       hexii(bool): Set to True, if a hexii-dump should be returned instead of a hexdump.
+      begin(int):  Offset of the first byte to print in the left column
+      style(dict): Color scheme to use.
+      highlight(iterable): Byte values to highlight.
 
     Returns:
       A hexdump-dump in the form of a string.
 """
+    style     = style or {}
+    highlight = highlight or []
+
+    for b in highlight:
+        if isinstance(b, str):
+            b = ord(b)
+        style['%02x' % b] = text.white_on_red
+    _style = style
+    style = default_style.copy()
+    style.update(_style)
+
+    skipping    = False
     lines       = []
     last_unique = ''
     byte_width  = len('00 ')
     column_sep  = '  '
-    line_fmt    = '%%(offset)08x  %%(hexbytes)-%is |%%(printable)s|' % (len(column_sep)+(width*byte_width))
+    line_fmt    = '%%(offset)08x  %%(hexbytes)-%is │%%(printable)s│' % (len(column_sep)+(width*byte_width))
+    spacer      = ' '
+    marker      = (style.get('marker') or (lambda s:s))('│')
 
     if hexii:
         column_sep = ''
-        line_fmt   = '%%(offset)08x  %%(hexbytes)-%is|' % (len(column_sep)+(width*byte_width))
+        line_fmt   = '%%(offset)08x  %%(hexbytes)-%is│' % (len(column_sep)+(width*byte_width))
+    else:
+        def style_byte(b):
+            hbyte = '%02x' % ord(b)
+            abyte = b if isprint(b) else '·'
+            if hbyte in style:
+                st = style[hbyte]
+            elif isprint(b):
+                st = style.get('printable')
+            else:
+                st = style.get('nonprintable')
+            if st:
+                hbyte = st(hbyte)
+                abyte = st(abyte)
+            return hbyte, abyte
+        cache = [style_byte(chr(b)) for b in range(256)]
 
     for line, chunk in enumerate(lists.group(width, s)):
         # If this chunk is the same as the last unique chunk,
         # use a '*' instead.
-        if skip and (last_unique == chunk):
-            if lines[-1] != '*':
-                lines.append('*')
+        if skip and (last_unique == chunk or sequential_lines(last_unique, chunk)):
+            last_unique = chunk
+            if not skipping:
+                yield '*'
+                skipping = True
             continue
 
         # Chunk is unique, save for next iteration
         last_unique = chunk
+        skipping = False
 
         # Cenerate contents for line
-        offset    = line*width
-        if not hexii:
-            hexbytes  = ''.join('%02x ' % ord(b) for b in chunk)
-            printable = ''.join(b if isprint(b) else '.' for b in chunk)
-        else:
-            hexbytes  = ''.join(_hexiichar(b) for b in chunk)
-            printable = ''
+        offset    = begin+line*width
+        hexbytes = ''
+        printable = ''
+        for i, b in enumerate(chunk):
+            if not hexii:
+                hbyte, abyte = cache[ord(b)]
+            else:
+                hbyte, abyte = _hexiichar(b), ''
 
-        # Insert column break in middle, for even-width lines
-        middle = (width/2)*byte_width
-        if len(hexbytes) > middle:
-            hexbytes = hexbytes[:middle] + column_sep + hexbytes[middle:]
+            if i % 4 == 3 and i < width - 1:
+                hbyte += spacer
+                abyte += marker
 
-        lines.append(line_fmt % {'offset': offset, 'hexbytes': hexbytes, 'printable': printable})
+            hexbytes += hbyte + ' '
+            printable += abyte
 
-    lines.append("%08x" % len(s))
-    return '\n'.join(lines)
+        if i + 1 < width:
+            delta = width - i - 1
+            hexbytes += ' ' * (byte_width * delta + (delta - 1) // 4)
+
+        line = line_fmt % {'offset': offset, 'hexbytes': hexbytes, 'printable': printable}
+        yield line
+
+    line = "%08x" % (len(s) + begin)
+    yield line
+
+def hexdump(s, width = 16, skip = True, hexii = False, begin = 0,
+            style = None, highlight = None):
+    return '\n'.join(hexdump_iter(s, width, skip, hexii, begin, style, highlight))
